@@ -10,6 +10,7 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <glib/gerror.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrange.h>
@@ -17,8 +18,6 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-about.h>
 #include <libgnome/gnome-help.h>
 #include <gtk/gtk.h>
 
@@ -40,6 +39,7 @@ extern void destroy_cache_hashTable (void);
 static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
 
 static gboolean position_auto_update=TRUE;
+static gboolean stopped_state=FALSE;
 static gboolean position_update_ignore_event=FALSE;
 static gchar* discid_prev;
 
@@ -184,11 +184,16 @@ play_cb (GtkButton *button,
 			end_track = -1;
 			endmsf = NULL;
 		} else {
-			end_track = status->track + 1;
-			msf.minute = 0;
-			msf.second = 0;
-			msf.frame = 0;
-			endmsf = &msf;
+			if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+				end_track = -1;
+				endmsf = NULL;	
+			} else {	
+				end_track = status->track + 1;
+				msf.minute = 0;
+				msf.second = 0;
+				msf.frame = 0;
+				endmsf = &msf;
+			}
 		}
 		if (gnome_cdrom_play (gcd->cdrom, status->track,
 				      &status->relative, end_track, endmsf, &error) == FALSE) {
@@ -209,6 +214,7 @@ play_cb (GtkButton *button,
 		break;
 
 	case GNOME_CDROM_AUDIO_COMPLETE:
+		stopped_state = FALSE;
 	case GNOME_CDROM_AUDIO_STOP:
 		msf.minute = 0;
 		msf.second = 0;
@@ -266,6 +272,8 @@ stop_cb (GtkButton *button,
 		return;
 	}
 
+	stopped_state = TRUE;
+
 	if (gcd->current_image == gcd->pause_image) {
 		AtkObject *aob;
 
@@ -592,26 +600,31 @@ status_ok (GnomeCD *gcd,
 		break;
 		
 	case GNOME_CDROM_AUDIO_COMPLETE:
-		if (gcd->cdrom->loopmode == GNOME_CDROM_LOOP) {
+		if (gcd->cdrom->loopmode == GNOME_CDROM_LOOP && stopped_state == FALSE) {
 			if (gcd->cdrom->playmode == GNOME_CDROM_WHOLE_CD) {
 				/* Around we go */
 				play_cb (NULL, gcd);
 			} else {
-				GnomeCDRomMSF msf;
+				GnomeCDRomMSF msf, *endmsf;
 				int start_track, end_track;
 				GError *error = NULL;
 				
-				/* CD has gone to the start of the next track.
-				   Track we want to loop is track - 1 */
-				start_track = status->track - 1;
-				end_track = status->track;
-				
+				/* CD is still in the same track */
 				msf.minute = 0;
 				msf.second = 0;
 				msf.frame = 0;
 				
+				start_track = status->track;
+				if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+					end_track = -1;
+					endmsf = NULL;
+				} else {
+					end_track = status->track + 1;
+					endmsf = &msf;
+				}
+				
 				if (gnome_cdrom_play (gcd->cdrom, start_track, &msf,
-						      end_track, &msf, &error) == FALSE) {
+						      end_track, endmsf, &error) == FALSE) {
 					gcd_warning ("%s", error);
 					g_error_free (error);
 					
@@ -750,6 +763,7 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 
         /* everything below are states where we're not playing */
 	case GNOME_CDROM_STATUS_NO_DISC:
+	case GNOME_CDROM_STATUS_EMPTY_DISC:
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
 			gcd->disc_info = NULL;
@@ -772,7 +786,10 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		gtk_widget_set_sensitive (gcd->eject_b, TRUE);
 
 		cd_display_clear (CD_DISPLAY (gcd->display));
-		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No disc"));
+		if (status->cd == GNOME_CDROM_STATUS_EMPTY_DISC)
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("Empty disc"));
+		else
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No disc")); 
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 
@@ -1231,8 +1248,13 @@ position_changed (GtkRange *range,
 	msf2.frame = 0;
 	
 	if (gcd->cdrom->playmode == GNOME_CDROM_SINGLE_TRACK) {
-		end_track = status->track + 1;
-		endmsf = &msf2;
+		if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+			end_track = -1;
+			endmsf = NULL;
+		} else {	
+			end_track = status->track + 1;
+			endmsf = &msf2;
+		}
 	} else {
 		end_track = -1;
 		endmsf = NULL;
