@@ -32,6 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <string.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -113,6 +114,7 @@ struct _GSRWindowPrivate {
 
 	int n_channels, bitrate, samplerate;
 	gboolean has_file;
+	gboolean saved;
 	gboolean dirty;
 	gboolean seek_in_progress;
 
@@ -616,10 +618,13 @@ do_save_file (GSRWindow *window,
 		if (result == GNOME_VFS_OK) {
 			g_object_set (G_OBJECT (window), "location", name, NULL);
 			priv->dirty = FALSE;
+			window->priv->saved = TRUE;
+			if ( GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "save_quit")))
+				gsr_window_close (window);
 		} else {
 			gchar *error_message;
  
-			error_message = g_strdup_printf (_("Could not save the file \" %s\""), gnome_vfs_result_to_string (result));
+			error_message = g_strdup_printf (_("Could not save the file \"%s\""), gnome_vfs_result_to_string (result));
 			show_error_dialog (GTK_WINDOW (window), error_message);
 			g_free (error_message);
 		}
@@ -660,6 +665,7 @@ file_chooser_save_response_cb (GtkDialog *file_chooser,
 	g_free (name);
 
  out:	
+	g_object_set_data (G_OBJECT (window), "save_quit", GPOINTER_TO_INT (NULL));
 	gtk_widget_destroy (GTK_WIDGET (file_chooser));
 }
 
@@ -765,6 +771,68 @@ run_mixer_cb (GtkAction *action,
 	g_free (mixer_path);
 }
 
+gboolean
+gsr_window_is_saved (GSRWindow *window)
+{
+	return window->priv->saved;
+}
+
+static void
+handle_confirmation_response (GtkDialog *dialog, gint response_id, GSRWindow *window)
+{
+	switch (response_id) {
+		case GTK_RESPONSE_YES:
+			g_object_set_data (G_OBJECT (window), "save_quit", GINT_TO_POINTER (TRUE));
+			file_save_as_cb (NULL, window);
+				break;
+
+		case GTK_RESPONSE_NO:
+			gsr_window_close (window);
+			break;
+
+		case GTK_RESPONSE_CANCEL:
+		default: 
+			break;
+	} 
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	return;
+}
+
+void 
+close_confirmation_dialog (GSRWindow *window)
+{
+	GtkDialog *confirmation_dialog;
+	char *msg;
+	AtkObject *atk_obj;
+
+	msg = g_strdup_printf (_("Save the changes to file \"%s\" before closing?"),
+			       window->priv->record_filename);
+
+	confirmation_dialog = gtk_message_dialog_new_with_markup (NULL,
+								  GTK_DIALOG_MODAL,
+						 		  GTK_MESSAGE_WARNING,
+								  GTK_BUTTONS_NONE,
+								  "<span weight=\"bold\" size=\"larger\">%s</span>",
+								  msg);
+
+	gtk_dialog_add_buttons (confirmation_dialog,
+			 	_("Close _without Saving"), GTK_RESPONSE_NO,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_SAVE, GTK_RESPONSE_YES, NULL);
+
+	gtk_window_set_title (GTK_WINDOW (confirmation_dialog), "");
+
+	g_signal_connect (G_OBJECT (confirmation_dialog), "response",
+			  G_CALLBACK (handle_confirmation_response), window);
+
+	atk_obj = gtk_widget_get_accessible (GTK_WIDGET (confirmation_dialog));
+	atk_object_set_name (atk_obj, _("Question"));
+
+	gtk_widget_show_all (GTK_WIDGET (confirmation_dialog));
+	g_free (msg);
+}
+
 static GtkWidget *
 make_title_label (const char *text)
 {
@@ -1028,7 +1096,11 @@ static void
 file_close_cb (GtkAction *action,
 	       GSRWindow *window)
 {
-	gsr_window_close (window);
+	if (! gsr_window_is_saved (window)) {
+		close_confirmation_dialog (window);
+	} else {
+		gsr_window_close (window);
+	}
 }
 
 static void
@@ -1058,14 +1130,17 @@ static void
 about_cb (GtkAction *action,
 	  GSRWindow *window)
 {
-	const char * const authors[] = {"Iain Holmes <iain@prettypeople.org>", NULL};
+	const char * const authors[] = {"Iain Holmes <iain@prettypeople.org>", 
+		"Ronald Bultje <rbultje@ronald.bitfreak.net>", 
+		"Johan Dahlin  <johan@gnome.org>", 
+		NULL};
 	const char * const documenters[] = {"Sun Microsystems", NULL};
  
 	gtk_show_about_dialog (GTK_WINDOW (window),
 			       "name", _("Sound Recorder"),
 			       "version", VERSION,
 			       "copyright", "Copyright \xc2\xa9 2002 Iain Holmes",
-			       "comments", _("A sound recorder for GNOME"),
+			       "comments", _("A sound recorder for GNOME\n gnome-media@gnome.org"),
 			       "authors", authors,
 			       "documenters", documenters,
 			       "logo-icon-name", "gnome-grecord",
@@ -1165,6 +1240,7 @@ record_cb (GtkAction *action,
 		      "location", priv->record_filename,
 		      NULL);
 	window->priv->len_secs = 0;
+	window->priv->saved = FALSE;
 	gst_element_set_state (priv->record->pipeline, GST_STATE_PLAYING);
 }
 
@@ -1634,37 +1710,37 @@ calculate_format_value (GtkScale *scale,
 	
 static const GtkActionEntry menu_entries[] =
 {
-	/* File menu */
+	/* File menu.  */
 	{ "File", NULL, N_("_File") },
-	{ "FileNew", GTK_STOCK_NEW, NULL, "<control>N",
+	{ "FileNew", GTK_STOCK_NEW, NULL, NULL,
 	  N_("Create a new sample"), G_CALLBACK (file_new_cb) },
-	{ "FileOpen", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
+	{ "FileOpen", GTK_STOCK_OPEN, NULL, NULL,
 	  N_("Open a file"), G_CALLBACK (file_open_cb) },
-	{ "FileSave", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
+	{ "FileSave", GTK_STOCK_SAVE, NULL, NULL,
 	  N_("Save the current file"), G_CALLBACK (file_save_cb) },
-	{ "FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<shift><control>S",
+	{ "FileSaveAs", GTK_STOCK_SAVE_AS, NULL, "<shift><control>S",
 	  N_("Save the current file with a different name"), G_CALLBACK (file_save_as_cb) },
 	{ "RunMixer", GTK_STOCK_EXECUTE, N_("Run _Mixer"), NULL,
 	  N_("Run the audio mixer"), G_CALLBACK (run_mixer_cb) },
-	{ "FileProperties", GTK_STOCK_PROPERTIES, N_("File _Information"), NULL,
+	{ "FileProperties", GTK_STOCK_PROPERTIES, NULL, "<control>I",
 	  N_("Show information about the current file"), G_CALLBACK (file_properties_cb) },
-	{ "FileClose", GTK_STOCK_CLOSE, NULL, "<control>W",
+	{ "FileClose", GTK_STOCK_CLOSE, NULL, NULL,
 	  N_("Close the current file"), G_CALLBACK (file_close_cb) },
-	{ "Quit", GTK_STOCK_QUIT, NULL, "<control>Q",
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL,
 	  N_("Quit the program"), G_CALLBACK (quit_cb) },
 
 	/* Control menu */
 	{ "Control", NULL, N_("_Control") },
-	{ "Record", GTK_STOCK_MEDIA_RECORD, N_("_Record"), "<control>R",
+	{ "Record", GTK_STOCK_MEDIA_RECORD, NULL, "<control>R",
 	  N_("Record sound"), G_CALLBACK (record_cb) },
-	{ "Play", GTK_STOCK_MEDIA_PLAY, N_("_Play"), "<control>P",
+	{ "Play", GTK_STOCK_MEDIA_PLAY, NULL, "<control>P",
 	  N_("Play sound"), G_CALLBACK (play_cb) },
-	{ "Stop", GTK_STOCK_MEDIA_STOP, N_("_Stop"), "<control>X",
+	{ "Stop", GTK_STOCK_MEDIA_STOP, NULL, "<control>X",
 	  N_("Stop sound"), G_CALLBACK (stop_cb) },
 
 	/* Help menu */
 	{ "Help", NULL, N_("_Help") },
-	{"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
+	{"HelpContents", GTK_STOCK_HELP, NULL, NULL,
 	 N_("Open the manual"), G_CALLBACK (help_contents_cb) },
 	{ "About", GTK_STOCK_ABOUT, NULL, NULL,
 	 N_("About this application"), G_CALLBACK (about_cb) }
@@ -2141,6 +2217,8 @@ gsr_window_new (const char *filename)
 		window->priv->working_file = g_strdup (filename);
 	}
 
+	window->priv->saved = TRUE;
+
 	gtk_window_set_default_size (GTK_WINDOW (window), 512, 200);
 
 	return GTK_WIDGET (window);
